/***************************************************************************
 *   Copyright (C) 2005 by Hector Blanco de Frutos                         *
 *   hectorblanco@neopontec.com                                            *
 *   Go to LICENSE file to read the license agreement                      *
 ***************************************************************************/

#include <string>
#include <stdlib.h>
#include <time.h>
#include <math.h>

#include "../base/engine.h"
#include "screens.h"
#include "../gametypes.h"



using namespace std;

extern SDL_Joystick *joy;

extern NNG_colors gamecolor;
extern SDL_Surface *screen;
extern NNG_EngineOptions game_options;
extern NNG_Sprite asteroid1;
extern NNG_SpriteBase asteroid1_base;
extern NNG_Sprite ship1;
extern NNG_SpriteBase ship1_base;
extern NNG_Sprite fire1[FIRES];
extern NNG_SpriteBase fire1_base;
extern NNG_Sprite enemies[ENEMIES];
extern NNG_SpriteBase enemybase;
extern NNG_Sprite explosion[ENEMIES];
extern NNG_SpriteBase explosion_base;
SDL_Surface *game_bg, *text, *game_panel;
TTF_Font *font_score=NULL;
//SDLFont *font2;
string str;
char str_tmp[256];

int score=0;

float scroll=0;           // How much has the screen scrolled
Uint32 elast;             // Remembers when the last enemy came out
Uint32 dtime;             // Remembers when you were resurrected
Uint8* keys;			  // We store the state of keys here


int bdraw[FIRES];		// Item of the fire1 array in use
int edraw[ENEMIES];       // Marks active enemies
int xdraw[ENEMIES];		  // Marks active explosions
int xlast[ENEMIES];		  // last time that explosion appeared
int espeed[ENEMIES];      // Speeds of each enemy
int elife[ENEMIES];       // Life of each enemy
int einfo[ENEMIES];       // The type of enemy - bouncing or not
int gameover=0, paused=0, changed=1;
int i;

float sdlgt=0;           // Stores the time that has passed
float dt=0;              // The time that has elapsed since
                         // the previous frame
int td=0,td2=0;          // Used when checking the time difference
                         // (how much time has passed since last frame)



// This function draws the scrolling background
void DrawScroll()
{
  // We first check, if the background has scrolled more than it's own
  // width. And if so, then we must draw 2 images on the screen:
  // the end of one scrolling background and the start of another.
  if(scroll>5864-800)
  {
    DrawIMG(screen, game_bg, 0,13,(int)(5864-scroll),230,(int)scroll,0);
    DrawIMG(screen, game_bg, (int)(5864-scroll-1),13,(int)(320-(5864-scroll)),230,0,0);
  // If not then we draw only one background
  } else {
    DrawIMG(screen, game_bg, 0,13,320,240,(int)scroll,0);
  }
}

// Add a bullet onto the screen
void addBullet()
{
  // We loop through all the spots for bullets
  for(i=0;i<FIRES;i++)
  {
    // If we find an empty spot
    if(bdraw[i]==0)
    {
      // Set the bullet's position
      fire1[i].set((int)ship1.getx()+13,(int)ship1.gety()+5);
      // and mark the spot
      bdraw[i]=1;
      return;
    }
  }
}

// Add an enemy onto the screen
void addEnemy()
{
  int k=0;
  // We loop through all the spots for enemies
  for(i=0;i<ENEMIES;i++)
  {
    // If the spot is free
    if(edraw[i]==0)
    {
      // Make one enemy out of three
      k=rand()%3;
      if(k== 0)
      {
        		
        enemies[i].setFrame(1);
        // mark him as red (moving up and down)
        einfo[i]=rand()%32+1;
      } else if(k==1){
        	// and the others purple (frame 0 in the sprite animation)
        	enemies[i].setFrame(0);
        	einfo[i]=0;
      	}else if(k==2){
        	enemies[i].setFrame(2);
        	einfo[i]=0;		
      }
      // we stop the animation on the enemy sprite for safety
      // (so that it wouldn's start blinking blue and purple all the time)
      enemies[i].stopAnim();
      // We give the enemy a random y coordinate so that each enemy
      // pops up at a different location...
      enemies[i].set(320,10+rand()%217);
      // We also give the enemy random speed and life values
      //printf("Enemy added %i at (%i,%i)\n",i,enemies[i].getx(),enemies[i].gety());
      if (k==2){
      	espeed[i]=rand()%5+1;
      	elife[i]=rand()%4+1;
      	if (espeed[i]<5){
      		espeed[i]=4;
      	}
      }else{
      	espeed[i]=rand()%2+1;
      	elife[i]=rand()%1+1;
      }
      // fill in the spot
      edraw[i]=1;
      return;
    }
  }
}

void addLive()
{
  // We loop through all the spots for enemies
  for(i=0;i<ENEMIES;i++)
  {
    // If the spot is free
    if(edraw[i]==0)
    {
      // Make one enemy out of two
      if(rand()%2 == 0)
      {
        // blue (frame 1 in the sprite animation)
        enemies[i].setFrame(1);
        // mark him as blue (moving up and down)
        einfo[i]=rand()%300+1;
      } else {
        // and the others purple (frame 0 in the sprite animation)
        enemies[i].setFrame(0);
        // mark him as purple
        einfo[i]=0;
      }
      // we stop the animation on the enemy sprite for safety
      // (so that it wouldn's start blinking blue and purple all the time)
      enemies[i].stopAnim();
      // We give the enemy a random y coordinate so that each enemy
      // pops up at a different location...
      enemies[i].set(320,5+rand()%230);
      // We also give the enemy random speed and life values
      printf("Enemy added %i at (%f,%f)\n",i,enemies[i].getx(),enemies[i].gety());
      espeed[i]=rand()%4+1;
      elife[i]=rand()%3+1;

      // fill in the spot
      edraw[i]=1;
      return;
    }
  }
}

// The main function...
void screen_game()
{
  int lives=2, ttime1=0, ttime2=0, fired=0;      // The score and the lives;
  srand(time(NULL));
  Uint8* keys;
  SDL_Rect rect1, rect2;
  
  
  game_bg = LoadIMG("art/game_bg_01.png");
  game_panel = LoadIMG("art/game_panel.png");
  gameover=0;
  paused=0;
  
  //FIX: (in 1.0.0). Score is always 0 when starting a game
  //score=0;
  //END FIX

  
  if (font_score==NULL){
	  font_score=TTF_OpenFont("fonts/FreeSans.ttf", 16);
  }
  //font2 = initFont("fonts/font2");
  
  
	//Mix_Music *music;         // This will store our music
	Mix_Chunk *shot;          // This will store our shot sound
	Mix_Chunk *explosound;       // This will store our "explosion" sound

	// Load sound and music
	// Load in the music
  	//music = Mix_LoadMUS("music/MUS_INGAME1.xm");
  	// And sound effects
  	shot = Mix_LoadWAV("sound/AUD_LASER01.wav");
  	explosound = Mix_LoadWAV("sound/AUD_EXPLOSI1.wav");
	
	// Initialize all the enemies
	for(i=0;i<ENEMIES;i++){
    	enemies[i].set(game_options.wres,0);
    	edraw[i]=0;
    	explosion[i].set(game_options.wres,0);
    	xdraw[i]=0;
	}
        
	rect1.x=0;
	rect1.y=0;
	rect1.w=game_options.wres;
	rect1.h=13;
	
	rect2.x=0;
	rect2.y=13;
	rect2.w=game_options.wres;
	rect2.h=game_options.hres-13;	

  // We set it's positon at 50,220
  ship1.set(64,(game_options.hres-13)/2);
  // And make it blink
  ship1.setSpeed(1);
  ship1.setFrame(0);
  ship1.draw();



  // Draw some scene elements
   DrawIMG(screen,game_panel,0,0);
   DrawScroll();
    DrawIMG(screen, game_panel, 0,0);
    //DrawIMG(screen, text, (game_options.wres-32), 0);
	
    sprintf(str_tmp,"%i",lives);
    str=string(str_tmp);
	SDL_FreeSurface(text);
    text=TTF_RenderText_Blended(font_score,str.c_str(), gamecolor.white);
	//drawString(screen,font2,300,1, str_tmp);
    DrawIMG(screen, text, 300, 1);
	SDL_FreeSurface(text);
	
	i=0;
	while (i<25){
       	SDL_SetAlpha(screen,SDL_SRCALPHA, i+10);
       	SDL_Flip(screen);
       	SDL_Delay(30);
       	i++;
	}
	SDL_SetAlpha(screen,SDL_SRCALPHA, 255);

// Loop while done equals true
  int done=0;
  while(done == 0)
  {
  	
    // move x*dt pixels.
    td2=SDL_GetTicks();
    dt=((float)(td2-td))*0.1;
    td=td2;
    	
    if(!paused)
    {
      	sdlgt+=dt*10;
    }
	paused=0;
    // Check if we have some interesting events...
    SDL_Event event;
    while(SDL_PollEvent(&event))
    {
      // If we must quit, then do so
      if(event.type == SDL_QUIT) { done = 1; }

      // Has a key been pressed down?
      if ((event.type == SDL_KEYDOWN)||(event.type == SDL_JOYBUTTONDOWN))
      {
        // If it was escape then quit
        if((event.key.keysym.sym == SDLK_ESCAPE)||(event.jbutton.button == GP2X_BUTTON_START))
        {
          gameover=1;
          //exit(0);
          done = 1;
          //TTF_CloseFont(font_score);
        }
        // If it was space and the game isn't paused nor over
		  
        
        if((event.key.keysym.sym == SDLK_p)||(event.jbutton.button == GP2X_BUTTON_SELECT))
        {
          // Backup the current time to restore it later, plus an add.
            ttime1=(int)sdlgt;
            paused=0;
          	screen_pause();
          	ttime2=SDL_GetTicks();
          	sdlgt=ttime1;
          	td=td+(ttime2-td2);
          	dt=0;
        }
		if(event.key.keysym.sym == SDLK_SPACE){
			// Then add a bullet
          	addBullet();
          	Mix_PlayChannel(0,shot,0);
          // And play a corresponding sound
			}
		//break;
		}
  	}
	// GP2X Movement buttons
	//if ((event.type==SDL_JOYBUTTONDOWN)){
	   if ((SDL_JoystickGetButton(joy, GP2X_BUTTON_UP))||(SDL_JoystickGetButton(joy, GP2X_BUTTON_UPLEFT))){
	   //if ((event.jbutton.button==GP2X_BUTTON_UP)||(event.jbutton.button==GP2X_BUTTON_UPLEFT)||(event.jbutton.button==GP2X_BUTTON_UPRIGHT)){
	   	  // Move the ship up
		  ship1.yadd((int)(-2*dt));
		  // Prevent the ship from going outside the screen
		  if(ship1.gety()<13) ship1.yset(13);
	   }else if ((SDL_JoystickGetButton(joy, GP2X_BUTTON_DOWN))||(SDL_JoystickGetButton(joy, GP2X_BUTTON_DOWNLEFT))||(SDL_JoystickGetButton(joy, GP2X_BUTTON_DOWNRIGHT))){
	   	  // Move the ship down
		  ship1.yadd((int)(2*dt));
		  // Prevent the ship from going outside the screen
		   if(ship1.gety()>225) ship1.yset(225);
	   }else if ((SDL_JoystickGetButton(joy, GP2X_BUTTON_LEFT))||(SDL_JoystickGetButton(joy, GP2X_BUTTON_UPLEFT))||(SDL_JoystickGetButton(joy, GP2X_BUTTON_DOWNLEFT))){
	   	  // Move the ship left
		  ship1.xadd((int)(-2*dt));
		  // Prevent the ship from going outside the screen
		  if(ship1.getx()<0) ship1.xset(0);
	   }else if ((SDL_JoystickGetButton(joy, GP2X_BUTTON_RIGHT))||(SDL_JoystickGetButton(joy, GP2X_BUTTON_UPRIGHT))||(SDL_JoystickGetButton(joy, GP2X_BUTTON_DOWNRIGHT))){
	   	  // Move the ship right
		  ship1.xadd((int)(2*dt));
	 	  // Prevent the ship from going outside the screen
		  if(ship1.getx()>305) ship1.xset(305);
	   }
	   if((fired==0)&&((event.key.keysym.sym == SDLK_SPACE && !paused)||(SDL_JoystickGetButton(joy, GP2X_BUTTON_A))))
        {
          // Then add a bullet
          addBullet();
          Mix_PlayChannel(0,shot,0);
		  fired=1;
          // And play a corresponding sound
        }

		// Void firing cointinuously
		if((event.type == SDL_JOYBUTTONUP)||(event.type == SDL_KEYUP)){
			if(event.jbutton.button==GP2X_BUTTON_A){
				fired=0;
			}
		}
	//}

    // Get the state of the keyboard keys
    keys = SDL_GetKeyState(NULL);

      // If the keys UP, DOWN, LEFT and RIGHT have been pressed
      if(keys[SDLK_UP])
      {
        // Move the ship up
        ship1.yadd((int)(-2*dt));
        // Prevent the ship from going outside the screen
        if(ship1.gety()<13) ship1.yset(13);
      }
      if(keys[SDLK_DOWN])
      {
        // Move the ship down
        ship1.yadd((int)(2*dt));
        // Prevent the ship from going outside the screen
        if(ship1.gety()>225) ship1.yset(225);
      }
      if(keys[SDLK_LEFT])
      {
        // Move the ship left
        ship1.xadd((int)(-2*dt));
        // Prevent the ship from going outside the screen
        if(ship1.getx()<0) ship1.xset(0);
      }
      if(keys[SDLK_RIGHT])
      {
        // Move the ship right
        ship1.xadd((int)(2*dt));
        // Prevent the ship from going outside the screen
        if(ship1.getx()>305) ship1.xset(305);
      }
    	// If we aren't playing music at the moment
    	/*
		if(!Mix_PlayingMusic())
    	{
      		// then start playing it.
      		//Mix_PlayMusic(music, 0);
    	}
		*/
	if (paused == 0){
	  // Loop through all the bullets
      for(i=0;i<FIRES;i++)
      {
        // If they are visible
        if(bdraw[i]==1)
        {
          // then move them 3.5*dt units right
          fire1[i].xadd(3*dt);
          // If they are outside the screen area
          if(fire1[i].getx()>340)
          {
            // hide them
            bdraw[i]=0;
          }
        }
      }


      // Loop through all the enemies
      for(i=0;i<ENEMIES;i++){
        // If they are visible
        if(edraw[i]==1){
          // Move them left (their speed+1)*0.5*dt units
          enemies[i].xadd((int)-((float)espeed[i]+1)*dt*0.5);
          // If the sprites are colored blue then
          if(einfo[i]>0){
            enemies[i].yset(
             ( cos((enemies[i].getx()+einfo[i])*0.0174532925) *100)+230);
          }
          // If the enemy moves out of the screen
          if(enemies[i].getx()<-16){
            // Get rid of it
            edraw[i]=0;
            enemies[i].set(game_options.wres,0);
            // and then move on to the next enemy
            continue;
          }
          // Check for bullet collisions
          for(int j=0;j<FIRES;j++){
            // If we have a bullet at that spot
            if(bdraw[j]==1){
              // Check if it collides with the enemy
              if(Sprite_Collide(fire1[j],enemies[i])){
                // If so then decrease the enemies life
                elife[i]--;
                
                // If the enemy is dead
                if(elife[i]<1){
                  // Then don't draw it anymore
                  edraw[i]=0;
                  xdraw[i]=1;
                  xlast[i]=(Uint32)sdlgt;
                  Mix_PlayChannel(1,explosound,0);
                  explosion[i].set(enemies[i].getx(),enemies[i].gety());
                  explosion[i].setSpeed(1);
                  explosion[i].startAnim();
                  
                  enemies[i].set(game_options.wres,0);
                  // Increase the players score by 10
                  score+=10;
                  // Get rid of the bullet
                  bdraw[j]=0;
                  // move on to the next enemy
                  continue;
                } else {
                  // If we only injured the enemy then increase
                  // the score by one
                  score+=5;
                  // Get rid of the bullet
                  bdraw[j]=0;
                }
              }
            }
          }
          // Check for player collisions
          // If the ship is vulnearable to attack - is not blinking
          // And the enemy collided with the ship then
          if(ship1.getSpeed()==0 && Sprite_Collide(ship1,enemies[i]))
          {
			if((enemies[i].gety()>game_options.hres)||(enemies[i].gety()<13)){
				edraw[i]=0;
			}else{
			    lives--;
			      xdraw[i]=1;
                  xlast[i]=(Uint32)sdlgt;
                  Mix_PlayChannel(1,explosound,0);
                  explosion[i].set((int)enemies[i].getx(),(int)enemies[i].gety());
                  explosion[i].setSpeed(1);
                  explosion[i].startAnim();
            	
            	//DEBUG:
            	//printf("Killed by %i at %f,%f\n",i,enemies[i].getx(),enemies[i].gety());
      	
			}
            // decrease our lives
           // if we are dead then
            if(lives<0)
            {
              // mark the game as over
              gameover=1;
              
              //FIX: (in 1.0.0) lives no longer shows a -1.
              lives=0;
              //END FIX
              
              // stop moving/checking the enemies,
              // eg: get out of this loop
              break;
            }
            // Get rid of the enemy
            edraw[i]=0;
            // If we didn't completely die then reset our position
            ship1.set(10,game_options.hres/2);
            // Make the ship blink (animate)
            ship1.setSpeed(1);
            ship1.startAnim();
            // Mark the time when we died
            dtime=(Uint32)sdlgt;
          }
          
        }
      }

      // If 3 seconds have passed since we died/were resurrected then
      if(sdlgt-dtime>3000)
      {
        // Stop the ships blinking animation
        ship1.stopAnim();
        ship1.setSpeed(0);
        // Show the frame of animation with the ship on it
        ship1.setFrame(0);
      }
	
	    for(i=0;i<FIRES;i++){
	    	if((sdlgt-elast>750)&&(xdraw[i]==1)){
	    		xdraw[i]=0;
	    		explosion[i].stopAnim();	    		
	    	}
	    }
      // If 3/4 seconds have passed since the last enemy came out
      if(sdlgt-elast>750)
      {
        // Add a new enemy
        addEnemy();
        
        // Mark the time when this enemy came out
        elast=(Uint32)sdlgt;
      }
      // Scroll the screen by dt units
      //scroll += 0.5*dt;
      // If we have scrolled more than the background image then
      // decrease the scrolled amount by the width of the background image
      if(scroll>=5864) scroll-=5864;
      
    // Draw the scene
	SDL_FillRect(screen,&rect2,0x010000);
  	DrawScroll();
  	
  for(i=0;i<FIRES;i++)
  {
    // And if we have a bullet at that spot, then draw it.
    if(bdraw[i]==1) {
    	fire1[i].draw();
    }
  }

	if (changed==1){
  		SDL_FillRect(screen,&rect1,0x010000);
  		//sprintf(str,"%i",score);
		sprintf(str_tmp,"%i",score);
  		str=string(str_tmp);
    	text=TTF_RenderText_Blended(font_score,str.c_str(), gamecolor.white);
    	DrawIMG(screen, game_panel, 0,0);
    	DrawIMG(screen, text, (274), -7);
		//drawString(screen,font2,274,-10, str_tmp);
    	//sprintf(str,"%i",lives);
		sprintf(str_tmp,"%i",lives);
    	str=string(str_tmp);
    	SDL_FreeSurface(text);
    	text=TTF_RenderText_Blended(font_score,str.c_str(), gamecolor.white);
		//drawString(screen,font2,238,-10, str_tmp);
    	DrawIMG(screen, text, 238, -7);
		SDL_FreeSurface(text);
		
	}
	

  
	  // Loop thorough all the spots for enemies
	 
  for(i=0;i<ENEMIES;i++){
    // And if we have an enemy at that spot, then draw him.
    //if(edraw[i]==1) {
    	enemies[i].draw();
    	if (xdraw[i]==1){
	    	explosion[i].draw();
    	}
    	//printf("Enemy %i at (%i,%i)\n",i,enemies[i].getx(),enemies[i].gety());
    //}
  }
	ship1.draw();
	SDL_Flip(screen);
	
	
	
    SDL_Delay(20);
    // Some other comprobations
    //If the game ends, go to game over screen.
    if (gameover==1){
    	lives=0;
  		printf("Game over. Your score is: %i.\n\n",score); 	
    	screen_gameover();
    	done=1;
    }
   }
  }
  
  // Fade out all the audio channels in 1 second
  //Mix_FadeOutChannel(-1, 1500);
  // and the music in 1 second
  //Mix_FadeOutMusic(750);
  // wait
  SDL_Delay(500);

  // Free the sounds
  //Mix_FreeMusic(music);
  Mix_FreeChunk(shot);
  Mix_FreeChunk(explosound);
  
  // Free some memory
  SDL_FreeSurface(game_bg);
  SDL_FreeSurface(game_panel);
  //freeFont(font2);
  //TTF_CloseFont(font_score);
}

